; TONE.ASM -- Assembly language subroutine to sound a tone.
; From the book "dBASE III PLUS Programmer's Reference Guide" by Alan Simpson.

gate_port     equ     61H       ; PPI device
channel_2     equ     42H       ; timer device channel used for sound
channel_3     equ     43H       ; timer device control channel

code       segment         byte
        assume  cs:code

tone       proc    far
; convert the two numbers in the command string to binary, then push
; them on the stack:
        call atob               ; atob works off the command string using DS:BX
        cmp     dx, 0           ; don't allow a zero to be pushed
        jnz     p1              ; this would only result from a command error
        mov     dx, 1           ; push a 1 instead (very short duration)
p1:     push dx                 ; push the first parameter
        inc bx                  ; get the next number's 1st digit
        call atob               ; convert second parameter
        cmp     dx, 0           ; don't allow a zero to be pushed
        jnz     p2              ; this would only result from a command error
        mov     dx, 500         ; push 500 instead (a high note)
p2:     push dx                 ; push second parameter

        in      al, gate_port   ; enable sound generation
        or      al, 3
        out     gate_port, al

        mov     al, 0B6H        ; set timer chip to modify sound channel
        out     channel_3, al

        pop     ax              ; retrieve the second parameter
        out     channel_2, al   ; set sound channed for desired frequency
        mov     al, ah
        out     channel_2, al

        mov     ax, 40H         ; change DS to access system tick count
        mov     ds, ax
        mov     cx, ds:[6CH]    ; fetch low word of tick count from 40:6C
        pop     ax              ; retrieve the 1st parameter
        add     cx, ax          ; add ti to tick count
tite_loop:                      ; now loop until system tick count reaches
        cmp     cx, ds:[6CH]    ; the computed value in CX
        jnz     tite_loop

        in      al, gate_port   ; disable sound generation
        and     al, 0FCH
        out     gate_port, al

        ret                     ; Return to dBASE.
tone       endp

; routine to convert an ASCII string to a binary number:

atob    proc    near            ; enter here with BX pointing to first digit
        mov     cl, 3           ; to set up shift to multiply by eight
        sub     dx, dx          ; clear result register

; this is the main processing loop of subroutine atob:
mnlup:  mov  al, byte ptr [bx]  ; fetch the next character
        sub     ah, ah          ; clear high byte of ax
        cmp     ax, '0'         ; test for decimal digit
        jge     notneg          ; We are done if a value is found outside
        ret                     ; the range of 0-9.  Otherwise, we
notneg: cmp     ax, '9'         ; continue the conversion process
        jle     convert
        ret

convert: sub    ax, '0'         ; convert ASCII digit to binary value
        push    ax              ; save value while we multiply, below
; multiply partial result by tem: (fast execution method)
m10:    mov     ax, dx          ; save result register
        shl     dx, cl          ; multiply partial result by eight
        add     dx, ax          ; add previous value
        add     dx, ax          ; add it one last time, 8 + 1 + 1 = 10
        pop     ax              ; retrieve value of the latest character
        add     dx, ax          ; add it to partial result
        inc     bx              ; point to next character
        jmp     mnlup
atob endp

code         ends
                end
